// 有序循环双向链表的节点删除
// 要点：
//     通过循环找到循环双向链表中所有节点值与给定值相同的节点并删除
// 关键字：循环双向链表
#include <stdlib.h>
#define N 1000
typedef struct Node{
    int data;
    struct Node *r;
    struct Node *l;
}Node;  
//////////////////////////////////声明l，r域构成的结构为循环双向链表//////////////////////////////////
//@ shape r,l: c_dlist;

/////////////////////////////////////////声明逻辑变量////////////////////////////////////////////////
//@ logic Node* oldhead; //逻辑变量指针oldhead用于记录函数入口处链表的头节点
//@ logic int n; //逻辑变量n用于记录函数入口处链表的长度

/////////////////////////////////////////声明全局幽灵变量////////////////////////////////////////////
//@ ghost int j;


///////////////////////////////////////delete函数的验证/////////////////////////////////////////////
//函数协议：
//函数前条件：head指向长为n的有序循环双向链表
//函数后条件：返回值为空或者指向长为n-j的有序循环双向链表，j为删除的节点数

/*@ 
  requires    
      \c_dlist(head) && \length(head,r) == n && n >= 0 && n <= N && oldhead == head
      && \forall integer m:[0..n-2].head->(r:m)->data <= head->(r:m+1)->data
      ;
  assigns j, oldhead; //声明全局幽灵变量可变
  ensures     
      \result == \null && j == 0 && n == 0 && oldhead == \null
    ||  
      \dangling(\result) && \dangling(oldhead) && n == j && n >= 1 && n <= N
    ||
      \c_dlist(\result) && \length(\result,r) == n-j && n >= 1 && n <= N && j >= 0 && j <= n-1 
      && (oldhead == \result || \dangling(oldhead))
      && (\forall integer k:[0..n-j-1].\result->(r:k)->data != val) 
      //完成删除后的链表中不存在指定的数
      && \forall integer m:[0..n-j-2].\result->(r:m)->data <= \result->(r:m+1)->data 
      //完成删除后的链表仍为有序链表
      ;
*/
Node * delete(Node* head,int const val)/*@ shape Node forward: r;*/{
    Node* preptr;
    Node* ptr;
    //@ ghost int i;
    //@ ghost i = 0;
    //@ ghost j = 0;
    if(head == NULL){
        return NULL;
    }
    else{
        preptr = head;
        ptr = head->r;
        /*@ 
          loop invariant
          //1.循环入口
          n > 1 && n <= N && i == 0 && j == 0 && oldhead == head
          && preptr == head
          && \c_dlist_seg(head,ptr) && ptr == head->(r:1) 
          && \c_dlist_seg(ptr,head) && head == ptr->(r:n-1)
          && (\forall integer m:[0..n-2].head->(r:m)->data <= head->(r:m+1)->data)
        ||
          //2.循环中及出口
          n > 1 && n <= N && i >= 0 && i <= n-j-1 && j >= 0 && oldhead == head
          && \c_dlist_seg(head,preptr) && preptr == head->(r:i)
          && \c_dlist_seg(preptr,ptr) && ptr == preptr->(r:1)
          && \c_dlist_seg(ptr,head) && head == ptr->(r:n-1-j-i)
          && (\forall integer k:[1..i].head->(r:k)->data != val) 
          && (\forall integer m:[0..n-j-2].head->(r:m)->data <= head->(r:m+1)->data)
        ||
          //3.链表长度为1时，循环出口（无法进入循环）
          n == 1 && i == 0 && j == 0 && oldhead == head
          && \c_dlist(head) && preptr == head && preptr == ptr && \length(head,r) == 1
          ;
        */
        while(ptr != head){
            if(ptr->data == val){
                preptr->r = ptr->r;
                ptr->r->l = preptr;
                free(ptr);
                ptr = preptr->r; //@ ghost j = j + 1;
            }
            else{
                preptr = ptr;
                ptr = ptr->r; //@ ghost i = i + 1;
            }
        }
        if(head->data == val){
            preptr->r = head->r;
            head->r->l =preptr;
            head = head->r; //@ ghost j = j + 1;
            free(ptr);
        }
        return head;
    }
}